@@ -30,6 +30,7 @@
 #include <QCheckBox>
 #include <QFileInfo>
 #include <QStandardItemModel>
+#include <QThreadPool>
 
 #include <KDebug>
 #include <KConfigDialog>
@@ -50,6 +51,7 @@
 #include "configdialog.h"
 #include "picture.h"
 #include "slideshow.h"
+#include "imagescaler.h"
 
 Frame::Frame(QObject *parent, const QVariantList &args)
         : Plasma::Applet(parent, args),
@@ -74,10 +76,15 @@ Frame::Frame(QObject *parent, const QVariantList &args)
     m_updateTimer = new QTimer(this);
     m_updateTimer->setSingleShot(true);
     connect(m_updateTimer, SIGNAL(timeout()), this, SLOT(delayedUpdateSize()));
+
+    m_autoUpdateTimer = new QTimer(this);
+    m_autoUpdateTimer->setSingleShot(true);
+    connect(m_autoUpdateTimer, SIGNAL(timeout()), this, SLOT(reloadImage()));
 }
 
 Frame::~Frame()
 {
+    m_autoUpdateTimer->stop();
 }
 
 void Frame::init()
@@ -87,6 +94,8 @@ void Frame::init()
         frameReceivedUrlArgs = true;
     }
 
+    m_currentDay = QDate::currentDate();
+    
     m_slideNumber = 0;
     // Get config values
     KConfigGroup cg = config();
@@ -94,7 +103,6 @@ void Frame::init()
     m_frame = cg.readEntry("frame", false);
     m_shadow = cg.readEntry("shadow", true);
     m_roundCorners = cg.readEntry("roundCorners", false);
-    m_smoothScaling = cg.readEntry("smoothScaling", true);
     m_slideShow = cg.readEntry("slideshow", false);
     m_random = cg.readEntry("random", false);
     m_recursiveSlideShow = cg.readEntry("recursive slideshow", false);
@@ -102,15 +110,20 @@ void Frame::init()
     m_slideshowTime = cg.readEntry("slideshow time", 60); // default to 1 minute
     m_currentUrl = cg.readEntry("url", m_currentUrl);
     setAssociatedApplicationUrls(m_currentUrl);
-    m_potdProvider = cg.readEntry("potdProvider", "wcpotd");
+    m_potdProvider = cg.readEntry("potdProvider", QString());
     m_potd = cg.readEntry("potd", false);
+    m_autoUpdateIntervall = cg.readEntry("autoupdate time", 0);
 
     // Frame & Shadow dimensions
     m_frameOutline = 8;
     m_swOutline = 8;
 
     // Initialize the slideshow timer
-    connect(m_mySlideShow, SIGNAL(pictureUpdated()), this, SLOT(updatePicture()));
+    connect(m_mySlideShow, SIGNAL(pictureUpdated()), this, SLOT(scalePictureAndUpdate()));
+
+    connect(&m_waitForResize, SIGNAL(timeout()), this, SLOT(scalePictureAndUpdate()));
+    m_waitForResize.setSingleShot(true);
+    m_waitForResize.setInterval(200);
 
     initSlideShow();
     if (frameReceivedUrlArgs) {
@@ -157,18 +170,21 @@ void Frame::constraintsEvent(Plasma::Constraints constraints)
             m_frameOutline = 8;
             m_swOutline = 8;
             //Restore widget geometry to image proportions
-            resize(contentSizeHint());
-            emit appletTransformedItself();
+            QSizeF sizeHint = contentSizeHint();
+            if (sizeHint != geometry().size()) {
+                resize(sizeHint);
+                emit appletTransformedItself();
+            }
         }
         m_updateTimer->start(400);
     }
 
     if (constraints & Plasma::SizeConstraint) {
         //If on panel, keep geometry to 4:3 ratio
-        if(formFactor() == Plasma::Vertical) {
+        if (formFactor() == Plasma::Vertical) {
             setMinimumSize(QSizeF(0, contentsRect().width()/1.33));
             setMaximumSize(QSizeF(-1, contentsRect().width()/1.33));
-        } else if(formFactor() == Plasma::Horizontal) {
+        } else if (formFactor() == Plasma::Horizontal) {
             setMinimumSize(QSizeF(contentsRect().height()*1.33,0));
             setMaximumSize(QSizeF(contentsRect().height()*1.33,-1));
         } else {
@@ -191,14 +207,14 @@ void Frame::constraintsEvent(Plasma::Constraints constraints)
             m_slideFrame->setPos(x, y);
         }
 
-        updatePicture();
+        m_waitForResize.start();
         m_updateTimer->start(400);
     }
 }
 
 QSizeF Frame::contentSizeHint() const
 {
-    if (!m_pictureSize.isEmpty() && (formFactor() == Plasma::Planar || formFactor() == Plasma::MediaCenter)){
+    if (!m_pictureSize.isEmpty() && (formFactor() == Plasma::Planar || formFactor() == Plasma::MediaCenter)) {
         const qreal maxSize = qMax(contentsRect().width(), contentsRect().height());
         QSize size = m_pictureSize;
         size.scale(maxSize, maxSize, Qt::KeepAspectRatio);
@@ -217,6 +233,20 @@ QSizeF Frame::sizeHint(Qt::SizeHint which, const QSizeF &constraint) const
     }
 }
 
+void Frame::scalePictureAndUpdate()
+{
+    QImage img = m_mySlideShow->image();
+    ImageScaler *scaler = new ImageScaler(img, contentSizeHint().toSize());
+    connect(scaler, SIGNAL(scaled(const QImage&)), this, SLOT(imageScaled(const QImage&)));
+    QThreadPool::globalInstance()->start(scaler);
+}
+
+void Frame::imageScaled(const QImage &img)
+{
+    m_scaledImage = img;
+    updatePicture();
+}
+
 void Frame::updatePicture()
 {
     m_pictureSize = m_mySlideShow->image().size();
@@ -230,13 +260,15 @@ void Frame::updatePicture()
         setAssociatedApplicationUrls(m_mySlideShow->currentUrl());
     }
 
-    emit sizeHintChanged(Qt::PreferredSize);
-    resize(contentSizeHint());
+    if (sizeHint != geometry().size()) {
+        emit sizeHintChanged(Qt::PreferredSize);
+        resize(sizeHint);
+    }
 
     kDebug() << "Rendering picture";
 
     // create a QPixmap which can be drawn in paintInterface()
-    QPixmap picture = m_mySlideShow->image();
+    QPixmap picture = QPixmap::fromImage(m_scaledImage);
 
     if (picture.isNull()) {
         return;
@@ -256,7 +288,7 @@ void Frame::updatePicture()
     QSizeF scaledSize = frameRect.size();
     scaledSize.scale(frameRect.size(), Qt::KeepAspectRatio);
     frameRect = QRectF(QPoint(frameRect.x() + (frameRect.width() - scaledSize.width()) / 2,
-                      frameRect.y() + (frameRect.height() - scaledSize.height()) / 2), scaledSize);
+                              frameRect.y() + (frameRect.height() - scaledSize.height()) / 2), scaledSize);
 
     QRectF shadowRect;
     if (m_frame) {
@@ -311,7 +343,6 @@ void Frame::updatePicture()
     }
 
     // Picture
-
     // We save the painter here so the clipping only happens on the pixmap,
     // not on pixmap, frame, shadow, etc
     p->save();
@@ -320,12 +351,11 @@ void Frame::updatePicture()
     }
 
     // Respect the smoothScaling setting
-    p->setRenderHint(QPainter::SmoothPixmapTransform, m_smoothScaling);
+    p->setRenderHint(QPainter::SmoothPixmapTransform, true);
     // draw our pixmap into the computed rectangle
-    p->drawPixmap(frameRect.toRect(), m_mySlideShow->image());
+    p->drawPixmap(frameRect.toRect(), picture);
     p->restore();
 
-    p->setRenderHint(QPainter::SmoothPixmapTransform, true);
     // black frame
     if (m_frame) {
         p->setPen(QPen(Qt::black, 1, Qt::SolidLine, Qt::RoundCap, Qt::RoundJoin));
@@ -369,6 +399,11 @@ void Frame::updatePicture()
     p->end();
     delete p;
     update();
+
+    if (m_doAutoUpdate && !m_autoUpdateTimer->isActive()) {
+        kDebug() << "Autoupdate timer restarted:" << m_autoUpdateIntervall << "s";
+        m_autoUpdateTimer->start(m_autoUpdateIntervall * 1000);
+    }
 }
 
 void Frame::nextPicture()
@@ -446,7 +481,6 @@ void Frame::createConfigurationInterface(KConfigDialog *parent)
     connect(m_configDialog->imageUi.slideShowDirList, SIGNAL(currentRowChanged(int)), this, SLOT(updateButtons()));
 
     m_configDialog->setRoundCorners(m_roundCorners);
-    m_configDialog->setSmoothScaling(m_smoothScaling);
     m_configDialog->setShadow(m_shadow);
     m_configDialog->setShowFrame(m_frame);
     m_configDialog->setFrameColor(m_frameColor);
@@ -462,14 +496,18 @@ void Frame::createConfigurationInterface(KConfigDialog *parent)
     m_configDialog->imageUi.randomCheckBox->setCheckState(m_random ? Qt::Checked : Qt::Unchecked);
     m_configDialog->imageUi.recursiveCheckBox->setCheckState(m_recursiveSlideShow ? Qt::Checked : Qt::Unchecked);
 
-    m_configDialog->imageUi.potdComboBox->setCurrentIndex(m_configDialog->imageUi.potdComboBox->findData(m_potdProvider));
-
+    if (!m_potdProvider.isEmpty())
+        m_configDialog->imageUi.potdComboBox->setCurrentIndex(m_configDialog->imageUi.potdComboBox->findData(m_potdProvider));
+    else 
+        m_configDialog->imageUi.potdComboBox->setCurrentIndex(0);
+    
     m_configDialog->setCurrentUrl(m_currentUrl);
     m_configDialog->imageUi.slideShowDirList->clear();
     m_configDialog->imageUi.slideShowDirList->addItems(m_slideShowPaths);
     m_configDialog->imageUi.removeDirButton->setEnabled(!m_slideShowPaths.isEmpty());
     m_configDialog->imageUi.slideShowDelay->setTime(QTime(m_slideshowTime / 3600, (m_slideshowTime / 60) % 60, m_slideshowTime % 60));
     m_configDialog->previewPicture(m_mySlideShow->image());
+    m_configDialog->imageUi.autoUpdateTime->setTime(QTime(m_autoUpdateIntervall / 3600, (m_autoUpdateIntervall / 60) % 60));
 }
 
 void Frame::configAccepted()
@@ -478,8 +516,6 @@ void Frame::configAccepted()
     // Appearance
     m_roundCorners = m_configDialog->roundCorners();
     cg.writeEntry("roundCorners", m_roundCorners);
-    m_smoothScaling = m_configDialog->smoothScaling();
-    cg.writeEntry("smoothScaling", m_smoothScaling);
     m_shadow = m_configDialog->shadow();
     cg.writeEntry("shadow", m_shadow);
     m_frame = m_configDialog->showFrame();
@@ -519,6 +555,12 @@ void Frame::configAccepted()
     m_slideshowTime = timerTime.second() + timerTime.minute() * 60 + timerTime.hour() * 3600;
     cg.writeEntry("slideshow time", m_slideshowTime);
 
+    m_autoUpdateTimer->stop();
+
+    QTime AutoUpdateTimer = m_configDialog->imageUi.autoUpdateTime->time();
+    m_autoUpdateIntervall = AutoUpdateTimer.minute() * 60 + AutoUpdateTimer.hour() * 3600;
+    cg.writeEntry("autoupdate time", m_autoUpdateIntervall);
+
     QString potdProvider = m_configDialog->imageUi.potdComboBox->itemData(m_configDialog->imageUi.potdComboBox->currentIndex()).toString();
 
     if ((wasPotd && !m_potd) || (m_potd && potdProvider != m_potdProvider)) {
@@ -539,34 +581,37 @@ void Frame::configAccepted()
 void Frame::stopPotd()
 {
     Plasma::DataEngine *engine = dataEngine("potd");
-    QDate mCurrentDate = QDate::currentDate();
-    const QString identifier = m_potdProvider + ':' + mCurrentDate.toString(Qt::ISODate);
+    const QString identifier = m_potdProvider + ':' + m_currentDay.toString(Qt::ISODate);
     engine->disconnectSource(identifier, m_mySlideShow);
+    m_autoUpdateTimer->stop();
 }
 
 void Frame::initSlideShow()
 {
     m_mySlideShow->setUpdateInterval(0);
+    m_doAutoUpdate = false;
 
     if (m_slideShow) {
         m_mySlideShow->setRandom(m_random);
         m_mySlideShow->setDirs(m_slideShowPaths, m_recursiveSlideShow);
         m_mySlideShow->setUpdateInterval(m_slideshowTime * 1000);
     } else if (m_potd) {
+        m_dateChangedTimer = new QTimer( this );//change picture at midnight
+        connect( m_dateChangedTimer, SIGNAL( timeout() ), this, SLOT( checkDayChanged() ) );
+        m_dateChangedTimer->start( 60 * 60 * 1000 ); // every hour
         Plasma::DataEngine *engine = dataEngine("potd");
-        QDate mCurrentDate = QDate::currentDate();
-        const QString identifier = m_potdProvider + ':' + mCurrentDate.toString(Qt::ISODate);
-
+        const QString identifier = m_potdProvider + ':' + m_currentDay.toString(Qt::ISODate);
         engine->connectSource(identifier, m_mySlideShow);
-
-//FIXME: why is there a manual kicking of the engine?        const Plasma::DataEngine::Data data = engine->query(identifier);
     } else { //no slideshow so no random stuff
         m_mySlideShow->setRandom(false);
         m_mySlideShow->setImage(m_currentUrl.url());
+
+        if (m_autoUpdateIntervall > 0) {
+            m_doAutoUpdate = true;
+        }
     }
-
-    updatePicture();
-
+    
+    scalePictureAndUpdate();
 }
 
 void Frame::dragEnterEvent(QGraphicsSceneDragDropEvent *event)
@@ -616,7 +661,6 @@ void Frame::hoverEnterEvent(QGraphicsSceneHoverEvent *event)
         checkSlideFrame();
         m_slideFrame->show();
     }
-
     Applet::hoverEnterEvent(event);
 }
 
@@ -626,15 +670,14 @@ void Frame::hoverLeaveEvent(QGraphicsSceneHoverEvent *event)
         checkSlideFrame();
         m_slideFrame->hide();
     }
-
     Applet::hoverLeaveEvent( event );
 }
 
 void Frame::checkSlideFrame()
 {
-    if (m_slideFrame)
+    if (m_slideFrame) {
         return;
-
+    }
     m_slideFrame = new Plasma::Frame( this );
     m_slideFrame->setZValue( 10 );
 
@@ -664,14 +707,13 @@ void Frame::checkSlideFrame()
 
 void Frame::paintInterface(QPainter *p, const QStyleOptionGraphicsItem *option, const QRect &rect)
 {
-    //kDebug() << "Paint!";
     Q_UNUSED(option)
     if (m_slideShow) {
         // temporarily suspend the slideshow to allow time for loading the image
         m_mySlideShow->setUpdateInterval(0);
     }
 
-    p->drawPixmap(rect, m_pixmap, rect);
+    p->drawPixmap(rect, m_pixmap);
 
     if (m_slideShow) {
         // unsuspend the slideshow to allow time for loading the image
@@ -681,8 +723,35 @@ void Frame::paintInterface(QPainter *p, const QStyleOptionGraphicsItem *option,
 
 void Frame::delayedUpdateSize()
 {
-    resize(contentSizeHint());
-    emit appletTransformedItself();
+    QSizeF sizeHint = contentSizeHint();
+    if (sizeHint != geometry().size()) {
+        resize(sizeHint);
+        emit appletTransformedItself();
+    }
 }
 
-#include "frame.moc"
\ No newline at end of file
+void Frame::checkDayChanged()
+{
+    if ( ( m_currentDay != QDate::currentDate() ) ) {
+        reloadImage();
+        //keep after reloadImage()
+        m_currentDay = QDate::currentDate();
+    }
+}
+
+void Frame::reloadImage()
+{
+    if (m_potd) {
+        Plasma::DataEngine *engine = dataEngine("potd");
+        //disconnect yesterday's source
+        QString identifier = m_potdProvider + ':' + m_currentDay.toString(Qt::ISODate);
+        engine->disconnectSource(identifier, m_mySlideShow);
+        //connect today's source
+        identifier = m_potdProvider + ':' + QDate::currentDate().toString(Qt::ISODate); 
+        engine->connectSource(identifier, m_mySlideShow);
+    } else {
+        m_mySlideShow->updateImage(m_currentUrl.url());
+    }
+}
+
+#include "frame.moc"
