@@ -14,6 +14,8 @@
 
 #include <qcolor.h>
 #include <qhash.h>
+#include <QString>
+#include <QColor>
 
 class QMenu;
 class QWidget;
@@ -43,19 +45,23 @@ class Kolourpicker : public Plasma::Applet
         void grabClicked();
         void historyClicked();
         void colorActionTriggered(QAction *act);
+	void colorActionTriggered(const QColor& color);
         void clearHistory(bool save = true);
         void installFilter();
+	void setDefaultColorFormat(QAction* act);
 
     private:
         void addColor(const QColor &color, bool save = true);
         void saveData(KConfigGroup &cg);
-
+	QString toLatex(const QColor& color);
         Plasma::ToolButton *m_grabButton;
-        Plasma::ToolButton *m_historyButton;
-        QMenu *m_historyMenu;
+        Plasma::ToolButton *m_configAndHistory;
+        QMenu *m_configAndHistoryMenu;
         QHash<QColor, QAction *> m_menus;
         QStringList m_colors;
+	QStringList m_colors_format;
         QWidget *m_grabWidget;
+	QString m_color_format;
 };
 
 inline uint qHash(const QColor &color)
